@@ -13,6 +13,25 @@ logger = create_logger(__name__) #pysts.db.mongodb.engine
 connect=mongoengine.connect
 Document=mongoengine.DynamicDocument
 
+def delete_dups(doc,unique_keys):
+    if isinstance(unique_keys,str):
+        unique_keys=[unique_keys]
+    pipeline=[
+        {
+            "$group": {
+                "_id": {x: f"${x}" for x in unique_keys},
+                "uniqueIds": { "$addToSet": "$_id" },
+                "count": { "$sum": 1 }
+            }
+        },
+        { "$match": { "count": { "$gt": 1 } } },
+        {"$project": {"name" : "$uniqueIds", "_id" : 0} }
+    ]
+    duplicates=doc.objects.aggregate(pipeline)
+    ids=[x for duplicate in duplicates for x in duplicate['name'][:-1]]
+    q=MyDoc.objects(id__in=ids)
+    return q.delete()
+
 #Add to_df to querysets (also for property .file)
 def to_df(self,exclude=None):
     if exclude is None: exclude=['_id']
@@ -77,38 +96,23 @@ def update_or_create(self,query=None,*args,files=None,unique_keys=None,**kwargs)
 
     elif len(query)>0 and len(updates)==1 and '$set' in updates: #Bulk insert many
         setq=updates['$set']
-        ops=[];combined_query={'$or':[]}
-        result_ids=[]
         if unique_keys is None:
             unique_keys=[key for key,val in query[0].items() if type(val) not in [list,tuple,np.ndarray]]
 
-        deleted_count=0
         total_queries=len(query)
-        for query_i, cur_query in enumerate(query):
-            ops.append({**cur_query,**setq})
-            cur_query_unique_keys={key:val for key,val in cur_query.items() if key in unique_keys}
-            if len(cur_query_unique_keys)>0:
-                combined_query['$or'].append(cur_query_unique_keys)
-            if (query_i+1)%1000 == 0:
-                diff_t = int((datetime.now() - start_t).total_seconds())
-                logger.debug(f'update_or_create: Iteration {query_i+1} of {total_queries} for insert_many (with updates) [START]: Total seconds = {diff_t-last_diff_t}')
-                deleted_count += db_collection.delete_many(combined_query).deleted_count
-                diff_t = int((datetime.now() - start_t).total_seconds())
-                logger.debug(f'update_or_create: Iteration {query_i+1} of {total_queries} for insert_many (with updates) [DELETE]: Total seconds = {diff_t-last_diff_t}')
-                result=db_collection.insert_many(ops)
-                result_ids.extend(result.inserted_ids)
-                ops=[];combined_query={'$or':[]}
-                diff_t = int((datetime.now() - start_t).total_seconds())
-                logger.debug(f'update_or_create: Iteration {query_i+1} of {total_queries} for insert_many (with updates) [COMPLETE]: Total seconds = {diff_t-last_diff_t}')
+        ops=[{**cur_query,**setq} for cur_query in query]
 
-        if len(combined_query['$or'])>0:
-            deleted_count += db_collection.delete_many(combined_query).deleted_count
-        logger.debug(f'update_or_create: Deleted {deleted_count} documents')
+        ids=[]
         if len(ops)>0:
+            #Insert new docs
             result=db_collection.insert_many(ops)
-            result_ids.extend(result.inserted_ids)
+            ids=result.inserted_ids
+            #delete duplicates
+            if len(unique_keys)>0:
+                deleted_count=delete_dups(self._document,unique_keys)
+                if deleted_count>0:
+                    logger.debug(f'update_or_create: During insert_many (with updates) - deleted {deleted_count} duplicate documents based on the keys: {unique_keys}')
 
-        ids=result_ids
         msg='insert_many (with updates)'
     else:
         raise Exception(f'Nothing to update or create: query={query}; and updates={updates}')
